/*****************************************************************************
 * XMClib v2.2.0 - XMC Peripheral Driver Library
 *
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 ****************************************************************************/

/**
 * @file
 * @date 15 April, 2017
 * @version 1.0.0
 *
 * @brief CCU4 concatenation demo example
 *
 * The example show how to concatenate to CCU4x slice timers.
 * The example provides an API (see timer.h) to treat both slices as a 32bit timer.
 * The timer has a resolution of 1/fCPU
 *
 * In the example LED1 toggles at a frequency of 1Hz (polling the timer) and LED2 toogles at a frequency of 2Hz (using the user callback)
 *
 * History <br>
 *
 * Version 1.0.0 Initial <br>
 *
 */

/*********************************************************************************************************************
 * HEADER FILES
 ********************************************************************************************************************/
#include "xmc_gpio.h"
#include "timer.h"

/*********************************************************************************************************************
 * MACROS
 ********************************************************************************************************************/
#define LED1 P5_9
#define LED2 P5_8

void timer_cb(void)
{
  XMC_GPIO_ToggleOutput(LED2);

  TIMER_Stop();
  TIMER_Clear();
  TIMER_Start(288000000U);
}

int main(void)
{
  XMC_GPIO_SetMode(LED1, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetMode(LED2, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);

  TIMER_Init(0);

  TIMER_Start(288000000U);

  while(1U)
  {
    if (TIMER_GetTime() > 144000000U)
    {
      XMC_GPIO_SetOutputHigh(LED1);
    }
    else
    {
      XMC_GPIO_SetOutputLow(LED1);
    }
  }
}
